#
# application that helps to teach the concepts of probability density function
# 
# 
# License: Creative Commons: CC BY-SA
#

library(shiny)
library(DT)
library(ggplot2)
library(RcmdrMisc)

data(diamonds)
npoints <- length(diamonds$price)

# superior limit of the histogram
maxBinLimit <- ceiling(max(diamonds$price)/1000)*1000

server <- function(input, output) {

# server code for the sample histograms
    
    # sample size    
    sampleSize <- reactive ({
        input$sampleSize
    })
    
    # number of bins in the histograms  
    sampleBins <- reactive({
        width <- maxBinLimit/input$histogramBins;
        seq(0, maxBinLimit, width)
    })
    
    # rendering the relative frequency histogram of the full dataset
    output$fullHistogram <- renderPlot({
        Hist(diamonds$price, scale = "percent", freq = TRUE, breaks = sampleBins(), ylab="Relative Frequency (%)", main="Full dataset histogram", xlab = "Diamond Price")
    })
    
    # rendering the relative frequency histogram of a sample of the dataset
    output$sampleHistogram <- renderPlot({
        y <- sample(diamonds$price, sampleSize())
        Hist(y, scale = "percent", freq = TRUE, breaks = sampleBins(), ylab="Relative Frequency (%)", main="Sampled dataset histogram", xlab = "Diamond Price")
    })    
    
#
# server code for the relative frequency x density histograms
#
    
    # building the vector of bin limits from the textbox input
    breaks <- reactive({
      (as.numeric(strsplit(input$histogramBreaks, ",")[[1]]))*1000
    })
    
    # rendering the relative frequency histogram with equals bins   
    output$equalBinsHistogram <- renderPlot({
        # setting the width of the bins to the minimum width in the vector of limits from the textbox
        width <- max(breaks())
        for (i in 1:(length(breaks())-1)) {
            if (breaks()[i+1]-breaks()[i] < width) {
                width <- breaks()[i+1]-breaks()[i]
            }
        }
        seq(0, max(breaks()), width)
        Hist(diamonds$price, scale = "percent", freq = TRUE, breaks = seq(0, max(breaks()), width),
              main = "Relative Frequency with Equal Bins", xlab = "Diamond Price", ylab = "Relative Frequency (%)")
    })    

    # rendering relative frequency histogram with unequals bins  
    output$unequalBinsRelFreqHistogram <- renderPlot({
        Hist(diamonds$price, scale = "percent", freq = TRUE, breaks = breaks(),
           main = "Relative Frequency with Unequal Bins", xlab = "Diamond Price", ylab = "Relative Frequency (%)")
    })    
    
    # rendering the density frequency histogram with unequals bins 
    output$unequalBinsDensityHistogram <- renderPlot({
        Hist(diamonds$price, breaks = breaks(),
           main = "Frequency Density with Unequal Bins", xlab = "Diamond Price", ylab = "Frequency Density")
        if (input$adjustDistribution == TRUE) {
          curve(dgamma(x, shape = 1.15, scale = 3500), xlim = c(0, maxBinLimit), col = "red", lwd = "2", add = TRUE)
        }
    })    

    #
    # server code for the relative frequency x density histograms
    #
    
    # type of distribution    
    distribution <- reactive ({
        input$dist
    })
    
    # approximate number of bins in the histograms
    bins <- reactive({
        input$breaksDensity
    })
    
    # number of samples taken from the distribution
    n <- reactive ({
        input$n
    })
    
#
#  Generates n values from the distribution specified by the user
#  Comment the set.seed command if you do not want reproducible results.    
#    
    y <- reactive ({
        set.seed(333)
        switch(
            distribution(),
            "Normal" = {
                rnorm(n(), mean = input$mean, sd = input$stdev)
            },
            "Gamma" = {
                rgamma(n(), scale = input$scale, shape = input$shape)
            },
            "Uniform" = {
                runif(n(), min = input$minUnif, max = input$maxUnif)
            }
        )
    })

    xlimits <- reactive ({
        switch(
            distribution(),
            "Normal" = {
                c(input$mean - 4 * input$stdev, input$mean + 4 * input$stdev)
            },
            "Gamma" = {
                if (input$shape > 4) {
                    c(0, input$scale * input$shape)
                } else {
                 c(0, 8 * input$scale * input$shape)
                }
            },
            "Uniform" = {
                c(input$minUnif - 2, input$maxUnif + 2)
            }
        )
    })
    
    # generates the histogram from the y sample and the number of bins
    h <- reactive({
        hist(y(), breaks = bins(), plot = FALSE)
    })

    # calculate the maximum value of the relative frequency histogram for 10 bins
    # to be used for the y-axis scale in the relative frequency histograms
    maxRelFreqH <- reactive ({
        tempHist <- hist(y(), breaks = 10, plot = FALSE)
        max(tempHist$counts) / n()
    })
#
# generates the table containing the frequency count, relative frequency, 
# frequency density, and cumulative frequency density of the six bins around
# the bin with maximum frequency
#
    freq_table <- reactive ({
        max <- max(h()$counts)
        indmax <- which(h()$counts == max)
        table <- matrix(nrow = 0, ncol = 4)
        table <- rbind(table, c(" ... ", " ...", "...", "..."))
        if (indmax > 4) {
            range <- (indmax-3):(indmax +2)
        } else {
            range <- 1:6
        }
        for (i in range) {
            bin <-     switch(
                distribution(),
                "Normal" = {
                    paste(sprintf("%.2f", h()$breaks[i]), "-", sprintf("%.2f", h()$breaks[i + 1]))
                },
                "Gamma" = {
                    paste(sprintf("%d", h()$breaks[i]), "-", sprintf("%d", h()$breaks[i + 1]))
                },
                "Uniform" = {
                    paste(sprintf("%.2f", h()$breaks[i]), "-", sprintf("%.2f", h()$breaks[i + 1]))
                }
            )
            relativeFrequency <- h()$counts[i] / n()
            relFreqString <- paste(sprintf("%.6f", relativeFrequency))
            relFreqDensityString <- paste(sprintf("%.5f", h()$density[i]))
            cumRelFreq <- cumsum(h()$counts)[i] / n()
            cumFreqString <- paste(sprintf("%.4f", cumRelFreq))
            text <- c(bin, relFreqString, relFreqDensityString, cumFreqString)
            table <- rbind(table, text)
        }
        table <- rbind(table, c(" ... ", " ...", "...", "..."))
        colnames(table) <- c("Bin", "Relative frequency", "Frequency density", "Cumulative frequency density")
        rownames(table) <- NULL
        table
    })

    # rendering the frequency table
    output$frequencyTable <-
        DT::renderDataTable({freq_table()}, 
                options = list(autoWidth = TRUE, rownames = FALSE, dom = 't',
                              rowCallback = JS("function(r,d) {$(r).attr('height', '30px')}"),
                              columnDefs = list(list(width = '20%', targets = 0),
                                                list(targets = 0:3, class = "dt-right"),
                                                list(width = '15%', targets = 1:3)))
        )
  
    # rendering the Relative Frequency Histogram  
    output$relativeFrequencyHistogram <- renderPlot({
        relh <- h()
        relh$counts <- relh$counts / n()
        plot(relh, xlim= xlimits(), ylim = c(0, maxRelFreqH()), ylab = "Relative Frequency", main = "Relative Frequency Histogram", xlab = "")
        box()
    })
    
    # rendering the Frequency Density Histogram  
    output$densityHistogram <- renderPlot({
        plot(h(), xlim= xlimits(), freq = FALSE, xlab = "", ylab = "Frequency Density", main = "Frequency Density Histogram")
        box()
    })
    
    # rendering the Cumulative Frequency Density Histogram  
    output$cumulativeFrequencyDensityHistogram <- renderPlot({
        cumh <- h()
        cumh$counts <- cumsum(cumh$counts) / n()
        plot(cumh, xlim= xlimits(), xlab = "", ylab = "Cumulative Frequency Density", main = "Cumulative Frequency Density Histogram")
        box()
     })

}


ui <- navbarPage(
    "Sample Histograms and Probability density Function",
    
    tabPanel("Sample Histograms",
             sidebarLayout(
                 sidebarPanel(
                     numericInput("sampleSize", label = "Sample size",
                         min = 20, max = 50000, value = 20
                     ),
                     sliderInput("histogramBins", label = "Number of bins",
                         min = 10, max = 1000, value = 10
                     ),                    
                     submitButton("Submit")
                 ),
                 
                 mainPanel(column(8, plotOutput("fullHistogram")),
                           column(8, plotOutput("sampleHistogram")))
             )),
    
    tabPanel("Relative Frequency x Density Histogram",
             sidebarLayout(
                 sidebarPanel(
                     textInput("histogramBreaks", label = "Bin limits (min = 0, max = 19 x 10e3)", 
                               value = "0, 1, 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 19"
                     ),
                     checkboxInput("adjustDistribution", "Adjust a distribution to the histogram", 
                                   value = FALSE),
                     submitButton("Submit")
                 ),
                 mainPanel(fluidRow(
                                column(5, plotOutput("equalBinsHistogram"))),
                           fluidRow(
                                column(5, plotOutput("unequalBinsRelFreqHistogram")),
                                column(5, plotOutput("unequalBinsDensityHistogram"))
                           ))
                )
    ),
    tabPanel("Probability Density Function",
             sidebarLayout(
                 sidebarPanel(
                     selectInput("dist", label = expression("Distribution"),
                         choices = c("Normal", "Uniform", "Gamma"),
                         selected = "Gamma"
                     ),
                     
                     conditionalPanel(
                         condition = "input.dist == 'Normal'",
                            numericInput("mean", "Mean",
                                min = 0, max = 100, value = 10
                            ),
                            numericInput("stdev", "Standard Deviation",
                                min = 0, max = 50, value = 1
                            )
                     ),
                     conditionalPanel(
                         condition = "input.dist == 'Uniform'",
                             numericInput("minUnif", "Minimum value",
                                 min = 0, max = 50, value = 10
                             ),
                             numericInput("maxUnif", "Maximum value",
                                 min = 11, max = 50, value = 30
                             )
                     ),
                     conditionalPanel(
                         condition = "input.dist == 'Gamma'",
                         numericInput("shape", "Shape",
                             min = 0.1, max = 10, value = 1.15
                         ),
                         numericInput("scale", "Scale",
                             min = 0.1, max = 5000, value = 3500
                         )
                     ),
                     numericInput(
                         "n", "Sample Size",
                         min = 1000, max = 40000000, value = 20000000
                     ),
                     numericInput(
                         "breaksDensity", "Number of bins in the histogram:", 
                         min = 10, max = 2000, value = 20
                     ),
                     submitButton("Submit")
                 ),
                 
                 mainPanel(fluidRow(
                     column(5, align = "center", div(
                         dataTableOutput("frequencyTable"), style = "font-size:90%"
                     )),
                     column(5, plotOutput("relativeFrequencyHistogram"))
                 ),
                 fluidRow(
                     column(5, plotOutput("densityHistogram")),
                     column(5, plotOutput("cumulativeFrequencyDensityHistogram"))
                 ))
             ))
)

shinyApp(ui = ui, server = server)